@@ -965,7 +965,7 @@ void KFDMemoryTest::BinarySearchLargestBuffer(int allocNode, const HsaMemFlags &
     int ret;
 
     HsaMemMapFlags mapFlags = {0};
-    HSAuint64 granularityMB = 128;
+    HSAuint64 granularityMB = highMB > 512 ? 128 : 16;
 
     /* Testing big buffers in VRAM */
     unsigned int * pDb = NULL;
@@ -1255,7 +1255,8 @@ TEST_F(KFDMemoryTest, MMBench) {
             memFlags.ui32.HostAccess = 0;
             memFlags.ui32.NonPaged = 1;
             /* Upper limit of buffer number to fit 90% vram size */
-            bufLimit = ((vramSizeMB << 20) * 9 / 10) / bufSize ;
+            bufLimit = ((vramSizeMB << 20) * 8 / 10) / bufSize ;
+
             if (bufLimit == 0)
                 continue; // skip when bufSize > vram
 
@@ -2030,6 +2031,10 @@ TEST_F(KFDMemoryTest, MMBandWidth) {
         RECORD(mcpWTime) << MMBANDWIDTH_KEY_PREFIX << "mcpWTime";
         RECORD(accessRTime) << MMBANDWIDTH_KEY_PREFIX << "accessRTime";
         RECORD(accessWTime) << MMBANDWIDTH_KEY_PREFIX << "accessWTime";
+
+        // skip slow tests
+        if (mcpRTime + mcpWTime + accessRTime + accessWTime > 5000000)
+            break;
     }
 
     munmap(tmp, tmpBufferSize);
