@@ -36,6 +36,8 @@ void KFDSVMRangeTest::SetUp() {
 
     m_pIsaGen = IsaGenerator::Create(m_FamilyId);
 
+    SVMSetXNACKMode();
+
     ROUTINE_END
 }
 
@@ -46,6 +48,8 @@ void KFDSVMRangeTest::TearDown() {
         delete m_pIsaGen;
     m_pIsaGen = NULL;
 
+    SVMRestoreXNACKMode();
+
     KFDBaseComponentTest::TearDown();
 
     ROUTINE_END
@@ -690,7 +694,7 @@ TEST_F(KFDSVMRangeTest, MigrateAccessInPlaceTest) {
         return;
     }
 
-    unsigned int BufferSize = 256 << 20;
+    unsigned int BufferSize = MIN(256ULL << 20, GetVramSize(defaultGPUNode) / 2);
     SDMAQueue sdmaQueue;
     ASSERT_SUCCESS(sdmaQueue.Create(defaultGPUNode));
 
@@ -1287,10 +1291,8 @@ TEST_F(KFDSVMRangeTest, ReadOnlyRangeTest) {
      */
     int pid = fork();
     if (pid == 0) {
-        if (hsaKmtOpenKFD() != HSAKMT_STATUS_SUCCESS) {
-            WARN() << "KFD open failed in child process" << std::endl;
-            exit(1);
-        }
+        TearDown();
+        SetUp();
     } else {
         int childStatus;
 
