@@ -128,6 +128,43 @@ bool is_dgpu() {
     return is_dgpu_dev;
 }
 
+bool hasPciAtomicsSupport(int node) {
+    /* If we can't get Node Properties, assume a lack of Atomics support */
+    HsaNodeProperties *pNodeProperties = new HsaNodeProperties();
+    if (hsaKmtGetNodeProperties(node, pNodeProperties)) {
+        LOG() << "Unable to get Node Properties for node " << node << std::endl;
+        return false;
+    }
+
+    /* APUs don't have IO Links, but support Atomic Ops by default */
+    if (pNodeProperties->NumCPUCores && pNodeProperties->NumFComputeCores)
+        return true;
+
+    HsaIoLinkProperties *IolinkProperties = new HsaIoLinkProperties[pNodeProperties->NumIOLinks];
+    if (hsaKmtGetNodeIoLinkProperties(node, pNodeProperties->NumIOLinks, IolinkProperties)) {
+        LOG() << "Unable to get Node IO Link Information for node " << node << std::endl;
+        return false;
+    }
+
+    /* Make sure we're checking GPU-to-CPU connection here */
+    for (int linkId = 0; linkId < pNodeProperties->NumIOLinks; linkId++) {
+        /* Make sure it's a CPU */
+        HsaNodeProperties *linkProps = new HsaNodeProperties();
+        if (hsaKmtGetNodeProperties(IolinkProperties[linkId].NodeTo, linkProps)) {
+            LOG() << "Unable to get connected device's IO Link information" << std::endl;
+            return false;
+        }
+        if (linkProps->NumCPUCores) {
+            /* IOLink flags are only valid if Override flag is set */
+            return (IolinkProperties[linkId].Flags.ui32.Override &&
+                   !IolinkProperties[linkId].Flags.ui32.NoAtomics32bit &&
+                   !IolinkProperties[linkId].Flags.ui32.NoAtomics64bit);
+        }
+    }
+
+    return false;
+}
+
 unsigned int FamilyIdFromNode(const HsaNodeProperties *props) {
     unsigned int familyId = FAMILY_UNKNOWN;
 
@@ -717,10 +754,8 @@ HSAKMT_STATUS RegisterSVMRange(HSAuint32 GPUNode, void *MemoryAddress,
     attrs[3].value = GPUNode;
 
     r = hsaKmtSVMSetAttr(MemoryAddress, SizeInBytes, nattr, attrs);
-    if (r) {
-        LOG() << "set range attrs failed" << std::endl;
+    if (r)
         return HSAKMT_STATUS_ERROR;
-    }
 
     return HSAKMT_STATUS_SUCCESS;
 }
@@ -734,10 +769,8 @@ HSAKMT_STATUS SVMRangeGetPrefetchNode(void *MemoryAddress, HSAuint64 SizeInBytes
     attr.value = 0;
 
     r = hsaKmtSVMGetAttr(MemoryAddress, SizeInBytes, 1, &attr);
-    if (r) {
-        LOG() << "get prefetch node failed" << std::endl;
+    if (r)
         return HSAKMT_STATUS_ERROR;
-    }
 
     *PrefetchNode = attr.value;
 
@@ -753,10 +786,8 @@ HSAKMT_STATUS SVMRangePrefetchToNode(void *MemoryAddress, HSAuint64 SizeInBytes,
     attr.value = PrefetchNode;
 
     r = hsaKmtSVMSetAttr(MemoryAddress, SizeInBytes, 1, &attr);
-    if (r) {
-        LOG() << "set prefetch node failed" << std::endl;
+    if (r)
         return HSAKMT_STATUS_ERROR;
-    }
 
     return HSAKMT_STATUS_SUCCESS;
 }
@@ -770,10 +801,8 @@ HSAKMT_STATUS SVMRangeMapToNode(void *MemoryAddress, HSAuint64 SizeInBytes,
     attr.value = NodeID;
 
     r = hsaKmtSVMSetAttr(MemoryAddress, SizeInBytes, 1, &attr);
-    if (r) {
-        LOG() << "set map to node failed" << std::endl;
+    if (r)
         return HSAKMT_STATUS_ERROR;
-    }
 
     return HSAKMT_STATUS_SUCCESS;
 }
@@ -787,10 +816,8 @@ HSAKMT_STATUS SVMRangeMapInPlaceToNode(void *MemoryAddress, HSAuint64 SizeInByte
     attr.value = NodeID;
 
     r = hsaKmtSVMSetAttr(MemoryAddress, SizeInBytes, 1, &attr);
-    if (r) {
-        LOG() << "set map in place to node failed" << std::endl;
+    if (r)
         return HSAKMT_STATUS_ERROR;
-    }
 
     return HSAKMT_STATUS_SUCCESS;
 }
@@ -804,10 +831,8 @@ HSAKMT_STATUS SVMRangSetGranularity(void *MemoryAddress, HSAuint64 SizeInBytes,
     attr.value = Granularity;
 
     r = hsaKmtSVMSetAttr(MemoryAddress, SizeInBytes, 1, &attr);
-    if (r) {
-        LOG() << "set granularity failed" << std::endl;
+    if (r)
         return HSAKMT_STATUS_ERROR;
-    }
 
     return HSAKMT_STATUS_SUCCESS;
 }
